@@ -34,7 +34,7 @@ r'''############################################################################
 
 ###################################################################################
 
-__version__ = "25.7.18"
+__version__ = "25.7.20"
 
 print('=' * 70)
 print('MIDI Doctor')
@@ -612,9 +612,9 @@ def repair_flat_dynamics(notes,
 ###################################################################################
 
 def convert_bytes_in_nested_list(lst,
-                                encodingNone,
-                                errors='ignore',
-                                return_changed_events_count=False
+                                 encoding=None,
+                                 errors='ignore',
+                                 return_changed_events_count=False
                                 ):
 
     new_list = []
@@ -622,48 +622,46 @@ def convert_bytes_in_nested_list(lst,
 
     for item in lst:
         if isinstance(item, list):
-
-            result = convert_bytes_in_nested_list(item, 
-                                                  encoding, 
-                                                  errors, True
-                                                 )
-            
-            sublist, subcount = result
-            new_list.append(sublist)
-            ce_count += subcount
+            sub, sub_count = convert_bytes_in_nested_list(item, 
+                                                          encoding, 
+                                                          errors, 
+                                                          True
+                                                         )
+            new_list.append(sub)
+            ce_count += sub_count
 
         elif isinstance(item, bytes):
-            
             b = item
-            chosen_str = None
-            fallback_str = None
 
-            for enc in _TEXT_ENCODINGS:
-                try:
-                    s = b.decode(enc)
+            if encoding is None:
+                chosen = None
+                fallback = None
+
+                for enc in _TEXT_ENCODINGS:
+                    try:
+                        s = b.decode(enc)
+                        
+                    except (UnicodeDecodeError, LookupError):
+                        continue
+
+                    if s.encode(enc, errors=errors) == b:
+                        chosen = s
+                        break
+
+                    if fallback is None:
+                        fallback = s
+
+                if chosen is not None:
+                    decoded = chosen
+                    
+                elif fallback is not None:
+                    decoded = fallback
                     
-                except (UnicodeDecodeError, LookupError):
-                    continue
-
-                if s.encode(enc) == b:
-                    chosen_str = s
-                    break
-
-                if fallback_str is None:
-                    fallback_str = s
-
-            if chosen_str is not None:
-                decoded = chosen_str
-                
-            elif fallback_str is not None:
-                decoded = fallback_str
-                
-            else:
-                if encoding is None:
-                    decoded = b.decode('utf-8', errors)
-
                 else:
-                    decoded = b.decode(encoding, errors)
+                    decoded = b.decode('iso-8859-1', errors)
+
+            else:
+                decoded = b.decode(encoding, errors)
 
             new_list.append(decoded)
             ce_count += 1
@@ -678,7 +676,10 @@ def convert_bytes_in_nested_list(lst,
 
 ###################################################################################
 
-def unbyte_text_events(events, encoding='utf-8', return_unbyte_count=False):
+def unbyte_text_events(events,
+                       encoding=None,
+                       return_unbyte_count=False
+                      ):
 
     fixed_events = []
 
@@ -686,7 +687,10 @@ def unbyte_text_events(events, encoding='utf-8', return_unbyte_count=False):
     
     for e in events:
         if e[0] in text_events:
-            ne, cc = convert_bytes_in_nested_list(e, encoding=encoding, return_changed_events_count=True)
+            ne, cc = convert_bytes_in_nested_list(e, 
+                                                  encoding=encoding,
+                                                  return_changed_events_count=True
+                                                 )
             fixed_events.append(ne)
             uc_count += cc
 
@@ -709,7 +713,8 @@ def write_midi(ticks,
                return_midi_data=False,
                return_score=False,
                return_stats=False,
-               return_md5_hash=False
+               return_md5_hash=False,
+               force_utf8=False
               ):
 
     score = [ticks]
@@ -720,7 +725,10 @@ def write_midi(ticks,
         score.append(sorted(o+n, key=lambda x: x[1]))
 
     if write_midi:
-        midi_data = MIDI.score2midi(score)
+        midi_data = MIDI.score2midi(score, 
+                                    force_utf8=force_utf8
+                                   )
+        
         with open(file_name, 'wb') as fi:
             fi.write(midi_data)
             fi.close
@@ -728,7 +736,7 @@ def write_midi(ticks,
     out_data = {}
 
     if return_midi_data or return_md5_hash:
-        midi_data = MIDI.score2midi(score)
+        midi_data = MIDI.score2midi(score, force_utf8=force_utf8)
 
     if return_midi_data:
         out_data.update({'midi_data': midi_data})
@@ -750,7 +758,8 @@ def heal_midi(midi_file,
               output_dir='./healed_midis/',
               timings_divider=1,
               max_notes_dur=-1,
-              text_events_encoding='utf-8',
+              text_events_encoding=None,
+              force_utf8=False,
               quiet_vels_adj_threshold=56,
               flat_vels_adj_threshold=0.95,
               write_midi_to_file=True,
@@ -904,7 +913,8 @@ def heal_midi(midi_file,
                                    write_midi=False, 
                                    return_midi_data=True, 
                                    return_stats=True, 
-                                   return_score=True
+                                   return_score=True,
+                                   force_utf8=force_utf8
                                   )
 
             mdata = midi_dict['midi_data']
